@@ -11,7 +11,7 @@
 ;;;; in Text, HTML, and XML formats. This includes hyperlinking
 ;;;; capability and sub-objects.
 ;;;;
-;;;; $Id: mop.lisp 11447 2007-01-04 18:12:57Z kevin $
+;;;; $Id$
 ;;;;
 ;;;; This file is Copyright (c) 2000-2006 by Kevin M. Rosenberg
 ;;;; *************************************************************************
@@ -23,52 +23,52 @@
 (defclass hyperobject-class (standard-class)
   ( ;; slots initialized in defclass
    (user-name :initarg :user-name :type string :initform nil
-	      :accessor user-name
-	      :documentation "User name for class")
+              :accessor user-name
+              :documentation "User name for class")
    (user-name-plural :initarg :user-name-plural :type string :initform nil
-		     :accessor user-name-plural
-		     :documentation "Plural user name for class")
+                     :accessor user-name-plural
+                     :documentation "Plural user name for class")
    (default-print-slots :initarg :default-print-slots :type list :initform nil
-			:accessor default-print-slots
-			:documentation "Defaults slots for a view")
+                        :accessor default-print-slots
+                        :documentation "Defaults slots for a view")
    (description :initarg :description :initform nil
-		:accessor description
-		:documentation "Class description")
+                :accessor description
+                :documentation "Class description")
    (version :initarg :version :initform nil
-	    :accessor version
-	    :documentation "Version number for class")
+            :accessor version
+            :documentation "Version number for class")
    (closures :initarg :closures :initform nil
-	     :accessor closures
-	     :documentation "Closures to call on slot chnages")
+             :accessor closures
+             :documentation "Closures to call on slot chnages")
    (sql-name :initarg :sql-name :accessor sql-name :initform nil
-	     :documentation "SQL Name for this class")
+             :documentation "SQL Name for this class")
    (guid :initarg :guid :accessor guid :initform nil
-	 :documentation "ID string for this class")
+         :documentation "ID string for this class")
 
    ;;; The remainder of these fields are calculated one time
    ;;; in finalize-inheritence.
 
    (subobjects :initform nil :accessor subobjects
-	       :documentation
-	       "List of fields that contain a list of subobjects objects.")
+               :documentation
+               "List of fields that contain a list of subobjects objects.")
    (compute-cached-values :initform nil :accessor compute-cached-values
                          :documentation
                          "List of fields that contain a list of compute-cached-value objects.")
    (hyperlinks :type list :initform nil :accessor hyperlinks
-	       :documentation "List of fields that have hyperlinks")
+               :documentation "List of fields that have hyperlinks")
    (direct-rules :type list :initform nil :initarg :direct-rules
-		 :accessor direct-rules
-		 :documentation "List of rules to fire on slot changes.")
+                 :accessor direct-rules
+                 :documentation "List of rules to fire on slot changes.")
    (direct-views :type list :initform nil :initarg :direct-views
-		 :accessor direct-views
-		 :documentation "List of views")
+                 :accessor direct-views
+                 :documentation "List of views")
    (class-id :type integer :initform (+ (* 1000000 (get-universal-time)) (random 1000000))
-	     :accessor class-id
-	     :documentation "Unique ID for the class")
+             :accessor class-id
+             :documentation "Unique ID for the class")
    (default-view :initform nil :initarg :default-view :accessor default-view
-		 :documentation "The default view for a class")
+                 :documentation "The default view for a class")
    (documementation :initform nil :initarg :documentation
-		    :documentation "Documentation string for hyperclass.")
+                    :documentation "Documentation string for hyperclass.")
 
    ;; SQL commands
    (create-table-cmd :initform nil :reader create-table-cmd)
@@ -76,9 +76,9 @@
    (drop-table-cmd :initform nil :reader drop-table-cmd)
 
    (views :type list :initform nil :initarg :views :accessor views
-	  :documentation "List of views")
+          :documentation "List of views")
    (rules :type list :initform nil :initarg :rules :accessor rules
-	  :documentation "List of rules")
+          :documentation "List of rules")
    )
   (:documentation "Metaclass for Markup Language classes."))
 
@@ -90,7 +90,7 @@
    (lookup-keys :type list :initarg :lookup-keys :reader lookup-keys))
   (:documentation "subobject information")
   (:default-initargs :name-class nil :name-slot nil :lazy-class nil
-		     :lookup nil :lookup-keys nil))
+                     :lookup nil :lookup-keys nil))
 
 (defclass compute-cached-value ()
   ((name-class :type symbol :initarg :name-class :reader name-class)
@@ -101,10 +101,10 @@
    (lookup-keys :type list :initarg :lookup-keys :reader lookup-keys))
   (:documentation "subobject information")
   (:default-initargs :name-class nil :name-slot nil :lazy-class nil
-		     :lookup nil :lookup-keys nil))
+                     :lookup nil :lookup-keys nil))
 
 
-(defmethod print-object ((obj subobject) (s stream))
+(defmethod print-object ((obj subobject) s)
   (print-unreadable-object (obj s :type t)
     (format s "~S" (name-slot obj))))
 
@@ -116,9 +116,9 @@
     ;;    :type t
     :initform nil :initarg :lookup :reader lookup)
    (link-parameters :type list :initform nil :initarg :link-parameters
-		    :reader link-parameters)))
+                    :reader link-parameters)))
 
-(defmethod print-object ((obj hyperlink) (s stream))
+(defmethod print-object ((obj hyperlink) s)
   (print-unreadable-object (obj s :type t :identity t)
     (format s "~S" (name obj))))
 
@@ -135,37 +135,37 @@
 
 (defun remove-keyword-arg (arglist akey)
   (let ((mylist arglist)
-	(newlist ()))
+        (newlist ()))
     (labels ((pop-arg (alist)
-	     (let ((arg (pop alist))
-		   (val (pop alist)))
-	       (unless (equal arg akey)
-		 (setf newlist (append (list arg val) newlist)))
-	       (when alist (pop-arg alist)))))
+             (let ((arg (pop alist))
+                   (val (pop alist)))
+               (unless (equal arg akey)
+                 (setf newlist (append (list arg val) newlist)))
+               (when alist (pop-arg alist)))))
       (pop-arg mylist))
     newlist))
 
 (defun remove-keyword-args (arglist akeys)
   (let ((mylist arglist)
-	(newlist ()))
+        (newlist ()))
     (labels ((pop-arg (alist)
-	     (let ((arg (pop alist))
-		   (val (pop alist)))
-	       (unless (find arg akeys)
-		 (setf newlist (append (list arg val) newlist)))
-	       (when alist (pop-arg alist)))))
+             (let ((arg (pop alist))
+                   (val (pop alist)))
+               (unless (find arg akeys)
+                 (setf newlist (append (list arg val) newlist)))
+               (when alist (pop-arg alist)))))
       (pop-arg mylist))
     newlist))
 
 
 (defmethod shared-initialize :around ((class hyperobject-class) slot-names
                                         &rest initargs
-					&key direct-superclasses
+                                        &key direct-superclasses
                                         user-name sql-name name description
-					&allow-other-keys)
+                                        &allow-other-keys)
   ;(format t "ii ~S ~S ~S ~S ~S~%" initargs base-table direct-superclasses user-name sql-name)
   (let ((root-class (find-class 'hyperobject nil))
-	(vmc 'hyperobject-class)
+        (vmc 'hyperobject-class)
         user-name-plural user-name-str sql-name-str)
     ;; when does CLSQL pass :qualifier to initialize instance?
     (setq user-name-str
@@ -212,40 +212,41 @@
               (do-call-next-method direct-superclasses)
             (do-call-next-method          direct-superclasses #+nil (append (list root-class)
                                                                             direct-superclasses)))
-	(do-call-next-method direct-superclasses)))))
+        (do-call-next-method direct-superclasses)))))
 
 
 (defmethod finalize-inheritance :after ((cl hyperobject-class))
   "Initialize a hyperobject class. Calculates all class slots"
   (finalize-subobjects cl)
-  (finalize-compute-cached cl))
+  (finalize-compute-cached cl)
+  (init-hyperobject-class cl))
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (when (>= (length (generic-function-lambda-list
-		     (ensure-generic-function
-		      'compute-effective-slot-definition)))
-	    3)
+                     (ensure-generic-function
+                      'compute-effective-slot-definition)))
+            3)
     (pushnew :ho-normal-cesd cl:*features*))
 
     (when (>= (length (generic-function-lambda-list
-		       (ensure-generic-function
-			'direct-slot-definition-class)))
-	    3)
+                       (ensure-generic-function
+                        'direct-slot-definition-class)))
+            3)
       (pushnew :ho-normal-dsdc cl:*features*))
 
     (when (>= (length (generic-function-lambda-list
-		       (ensure-generic-function
-			'effective-slot-definition-class)))
-	      3)
+                       (ensure-generic-function
+                        'effective-slot-definition-class)))
+              3)
       (pushnew :ho-normal-esdc cl:*features*)))
 
 (defmethod direct-slot-definition-class ((cl hyperobject-class)
-					 #+ho-normal-dsdc &rest iargs)
+                                         #+ho-normal-dsdc &rest iargs)
   (declare (ignore iargs))
   (find-class 'hyperobject-dsd))
 
 (defmethod effective-slot-definition-class ((cl hyperobject-class)
-					    #+ho-normal-esdc &rest iargs)
+                                            #+ho-normal-esdc &rest iargs)
   (declare (ignore iargs))
   (find-class 'hyperobject-esd))
 
@@ -255,10 +256,10 @@
   (defmacro process-class-option (slot-name &optional required)
     #+lispworks
     `(defmethod clos:process-a-class-option ((class hyperobject-class)
-					     (name (eql ,slot-name))
-					     value)
+                                             (name (eql ,slot-name))
+                                             value)
       (when (and ,required (null value))
-	(error "hyperobject class slot ~A must have a value" name))
+        (error "hyperobject class slot ~A must have a value" name))
       (list name `',value))
     #+(or allegro sbcl cmu scl openmcl)
     (declare (ignore slot-name required))
@@ -267,10 +268,10 @@
   (defmacro process-slot-option (slot-name)
     #+lispworks
     `(defmethod clos:process-a-slot-option ((class hyperobject-class)
-					    (option (eql ,slot-name))
-					    value
-					    already-processed-options
-					    slot)
+                                            (option (eql ,slot-name))
+                                            value
+                                            already-processed-options
+                                            slot)
       (list* option `',value already-processed-options))
     #-lispworks
     (declare (ignore slot-name))
@@ -284,31 +285,31 @@
   (eval
    `(defclass hyperobject-dsd (standard-direct-slot-definition)
      (,@(mapcar #'(lambda (x)
-		    `(,(intern (symbol-name x))
-		      :initform nil))
-		*slot-options-no-initarg*)
+                    `(,(intern (symbol-name x))
+                      :initform nil))
+                *slot-options-no-initarg*)
       ,@(mapcar #'(lambda (x)
-		    `(,(intern (symbol-name x))
-		      :initarg
-		      ,(intern (symbol-name x) (symbol-name :keyword))
-		      :initform nil
-		      :accessor
-		      ,(intern (concatenate 'string
-					    (symbol-name :dsd-)
-					    (symbol-name x)))))
-		*slot-options*))))
+                    `(,(intern (symbol-name x))
+                      :initarg
+                      ,(intern (symbol-name x) (symbol-name :keyword))
+                      :initform nil
+                      :accessor
+                      ,(intern (concatenate 'string
+                                            (symbol-name :dsd-)
+                                            (symbol-name x)))))
+                *slot-options*))))
   (eval
    `(defclass hyperobject-esd (standard-effective-slot-definition)
      (,@(mapcar #'(lambda (x)
-		    `(,(intern (symbol-name x))
-		      :initarg
-		      ,(intern (symbol-name x) (symbol-name :keyword))
-		      :initform nil
-		      :accessor
-		      ,(intern (concatenate 'string
-					    (symbol-name :esd-)
-					    (symbol-name x)))))
-		(append *slot-options* *slot-options-no-initarg*)))))
+                    `(,(intern (symbol-name x))
+                      :initarg
+                      ,(intern (symbol-name x) (symbol-name :keyword))
+                      :initform nil
+                      :accessor
+                      ,(intern (concatenate 'string
+                                            (symbol-name :esd-)
+                                            (symbol-name x)))))
+                (append *slot-options* *slot-options-no-initarg*)))))
   ) ;; eval-when
 
 (defun intern-in-keyword (obj)
@@ -335,38 +336,38 @@
 
 
 (defmethod compute-effective-slot-definition :around ((cl hyperobject-class)
-						      #+ho-normal-cesd name
-						      dsds)
+                                                      #+ho-normal-cesd name
+                                                      dsds)
   (declare (ignore #+ho-normal-cesd name))
   (let ((esd (call-next-method)))
     (if (typep esd 'hyperobject-esd)
-	(compute-hyperobject-esd esd dsds)
-	esd)))
+        (compute-hyperobject-esd esd dsds)
+        esd)))
 
 (defun compute-hyperobject-esd (esd dsds)
   (let* ((dsd (car dsds)))
     (multiple-value-bind (sql-type sql-length)
-	(value-type-to-sql-type (dsd-value-type dsd))
+        (value-type-to-sql-type (dsd-value-type dsd))
       (setf (esd-sql-type esd) sql-type)
       (setf (esd-sql-length esd) sql-length))
     (setf (esd-user-name esd)
-	  (aif (dsd-user-name dsd)
+          (aif (dsd-user-name dsd)
                it
-	       (string-downcase (symbol-name (slot-definition-name dsd)))))
+               (string-downcase (symbol-name (slot-definition-name dsd)))))
     (setf (esd-sql-name esd)
-	  (aif (dsd-sql-name dsd)
-	       it
-	       (lisp-name-to-sql-name (slot-definition-name dsd))))
+          (aif (dsd-sql-name dsd)
+               it
+               (lisp-name-to-sql-name (slot-definition-name dsd))))
     (setf (esd-sql-name esd)
-	  (aif (dsd-sql-name dsd)
-	       it
-	       (lisp-name-to-sql-name (slot-definition-name dsd))))
+          (aif (dsd-sql-name dsd)
+               it
+               (lisp-name-to-sql-name (slot-definition-name dsd))))
     (dolist (name '(value-type print-formatter subobject hyperlink
                     hyperlink-parameters unbound-lookup
-		    description value-constraint indexed null-allowed
-		    unique short-description void-text read-only-groups
-		    hidden-groups unit disable-predicate view-type
-		    list-of-values compute-cached-value stored))
+                    description value-constraint indexed null-allowed
+                    unique short-description void-text read-only-groups
+                    hidden-groups unit disable-predicate view-type
+                    list-of-values compute-cached-value stored))
       (setf (slot-value esd name) (slot-value dsd name)))
     esd))
 
@@ -375,20 +376,20 @@
 SQL name"
   (unless (stringp lisp)
     (setq lisp
-	  (typecase lisp
-	    (symbol (symbol-name lisp))
-	    (t (write-to-string lisp)))))
+          (typecase lisp
+            (symbol (symbol-name lisp))
+            (t (write-to-string lisp)))))
   (do* ((len (length lisp))
-	(sql (make-string len))
-	(i 0 (1+ i)))
+        (sql (make-string len))
+        (i 0 (1+ i)))
       ((= i len) (string-upcase sql))
     (declare (fixnum i)
-	     (simple-string sql))
+             (simple-string sql))
     (setf (schar sql i)
-	  (let ((c (char lisp i)))
-	    (case c
-	      ((#\- #\$ #\+ #\#) #\_)
-	      (otherwise c))))))
+          (let ((c (char lisp i)))
+            (case c
+              ((#\- #\$ #\+ #\#) #\_)
+              (otherwise c))))))
 
 #+ho-normal-cesd
 (setq cl:*features* (delete :ho-normal-cesd cl:*features*))
@@ -430,28 +431,28 @@ SQL name"
 (defun value-type-to-sql-type (value-type)
   "Return two values, the sql type and field length."
   (let ((type (base-value-type value-type))
-	(length (when (consp value-type)
-		  (cadr value-type))))
+        (length (when (consp value-type)
+                  (cadr value-type))))
     (values
      (case type
        ((:char :character)
-	:char)
+        :char)
        (:varchar
-	:varchar)
+        :varchar)
        ((:fixnum :integer)
-	:integer)
+        :integer)
        (:long-integer
-	:long-integer)
+        :long-integer)
        (:boolean
-	:boolean)
+        :boolean)
        ((:float :single-float)
-	:single-float)
+        :single-float)
        (:double-float
-	:double-float)
+        :double-float)
        (:datetime
         :long-integer)
        (otherwise
-	:text))
+        :text))
      length)))
 
 ;;;; Class initialization function
@@ -462,29 +463,29 @@ SQL name"
 
 (defmethod slot-unbound ((class hyperobject-class) instance slot-name)
   (let ((lazy-reader
-	 (loop for super in (class-precedence-list class)
-	       as lazy-reader = (getf (gethash super *lazy-readers*) slot-name)
-	       when lazy-reader return it)))
+         (loop for super in (class-precedence-list class)
+               as lazy-reader = (getf (gethash super *lazy-readers*) slot-name)
+               when lazy-reader return it)))
     (if lazy-reader
-	(setf (slot-value instance slot-name)
-	      (if (atom lazy-reader)
-		  (make-instance lazy-reader)
-		  (apply (car lazy-reader)
-			 (loop for arg-slot-name in (cdr lazy-reader)
-			       collect (slot-value instance arg-slot-name)))))
-	;; No lazy reader -- defer to regular slot-unbound handling.
-	(call-next-method))))
+        (setf (slot-value instance slot-name)
+              (if (atom lazy-reader)
+                  (make-instance lazy-reader)
+                  (apply (car lazy-reader)
+                         (loop for arg-slot-name in (cdr lazy-reader)
+                               collect (slot-value instance arg-slot-name)))))
+        ;; No lazy reader -- defer to regular slot-unbound handling.
+        (call-next-method))))
 
 ;; The reader is a function and the reader-keys are slot names.  The slot is lazily set to
 ;; the result of applying the function to the slot-values of those slots, and that value
 ;; is also returned.
 (defun ensure-lazy-reader (cl class-name slot-name lazy-class reader
-			   &rest reader-keys)
+                           &rest reader-keys)
   (declare (ignore class-name))
   (setf (getf (gethash cl *lazy-readers*) slot-name)
     (aif lazy-class
-	 it
-	 (list* reader (copy-list reader-keys)))))
+         it
+         (list* reader (copy-list reader-keys)))))
 
 (defun remove-lazy-reader (class-name slot-name)
   (setf (getf (gethash (find-class class-name) *lazy-readers*) slot-name)
@@ -531,27 +532,27 @@ SQL name"
   "Calculate class documentation slot"
   (let ((*print-circle* nil))
     (setf (documentation cl 'type)
-	  (format nil "Hyperobject~A~A~A~A"
-		  (aif (user-name cl)
-		       (format nil ": ~A" it ""))
-		  (aif (description cl)
-		       (format nil "~%Class description: ~A" it) "")
-		  (aif (subobjects cl)
-		       (format nil "~%Subobjects:~{ ~A~}" (mapcar #'name-slot it)) "")
-		  (aif (default-print-slots cl)
-		       (format nil "~%Default print slots:~{ ~A~}" it) "")
-		  ))))
+          (format nil "Hyperobject~A~A~A~A"
+                  (aif (user-name cl)
+                       (format nil ": ~A" it ""))
+                  (aif (description cl)
+                       (format nil "~%Class description: ~A" it) "")
+                  (aif (subobjects cl)
+                       (format nil "~%Subobjects:~{ ~A~}" (mapcar #'name-slot it)) "")
+                  (aif (default-print-slots cl)
+                       (format nil "~%Default print slots:~{ ~A~}" it) "")
+                  ))))
 
 (defun finalize-hyperlinks (cl)
   (let ((hyperlinks '()))
     (dolist (esd (class-slots cl))
       (awhen (slot-value esd 'hyperlink)
-	     (push
-	      (make-instance 'hyperlink
-			     :name (slot-definition-name esd)
-			     :lookup it
-			     :link-parameters (slot-value esd 'hyperlink-parameters))
-	      hyperlinks)))
+             (push
+              (make-instance 'hyperlink
+                             :name (slot-definition-name esd)
+                             :lookup it
+                             :link-parameters (slot-value esd 'hyperlink-parameters))
+              hyperlinks)))
     ;; cmu/sbcl reverse class-slots compared to the defclass form
     ;; hyperlinks is already reversed from the dolist/push loop, so re-reverse on sbcl/cmu
     #-(or cmu sbcl) (setq hyperlinks (nreverse hyperlinks))
